import {
  Component,
  NO_ERRORS_SCHEMA,
  OnInit,
  ViewChild,
} from "@angular/core";
import {
  async,
  ComponentFixture,
  TestBed,
} from "@angular/core/testing";
import {
  FormBuilder,
  FormControl,
  FormGroup,
  ReactiveFormsModule,
} from "@angular/forms";
import {NoopAnimationsModule} from "@angular/platform-browser/animations";
import {MaterialModule} from "@app/material.module";
import {TestHelpersModule} from "../../../../../test-helpers/test-helpers.module";

import {SharedMultiSelectDefaultValuePresentational} from "./shared-multi-select-default-value.presentational";

@Component({
  selector: "dlcm-mock-shared-multi-select",
  template: `
      <form [formGroup]="mockForm">
          <dlcm-shared-multi-select-default-value #multiSelectComponent
                                                  [formControl]="firstNameFromControl"
                                                  [formControlDefault]="defaultFormControl"
                                                  [labelKey]="'label'"
                                                  [placeholder]="'placeholder'"
                                                  [list]="''"
                                                  [valueKey]="''">
          </dlcm-shared-multi-select-default-value>
      </form>`,
})
export class MockSharedSearchableMultiSelectPresentational implements OnInit {

  mockForm: FormGroup;
  firstNameFromControl: FormControl;
  defaultFormControl: FormControl;

  @ViewChild("multiSelectComponent", {static: false})
  readonly multiSelectComponent: SharedMultiSelectDefaultValuePresentational;

  constructor(private fb: FormBuilder) {
  }

  ngOnInit(): void {
    this.firstNameFromControl = new FormControl("");
    this.defaultFormControl = new FormControl("");
    this.mockForm = this.fb.group({
      firstName: this.firstNameFromControl,
      default: this.defaultFormControl,
    });
  }
}

describe("SharedMultiSelectDefaultValuePresentational", () => {
  let component: MockSharedSearchableMultiSelectPresentational;
  let fixture: ComponentFixture<MockSharedSearchableMultiSelectPresentational>;

  beforeEach(async(() => {
    TestBed.configureTestingModule({
      declarations: [SharedMultiSelectDefaultValuePresentational, MockSharedSearchableMultiSelectPresentational],
      imports: [ReactiveFormsModule, MaterialModule, TestHelpersModule, NoopAnimationsModule],
      schemas: [NO_ERRORS_SCHEMA],
    })
      .compileComponents();
  }));

  beforeEach(() => {
    fixture = TestBed.createComponent(MockSharedSearchableMultiSelectPresentational);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it("should create", () => {
    expect(component).toBeTruthy();
    expect(component.multiSelectComponent).toBeTruthy();
  });
});
