import {
  ChangeDetectionStrategy,
  Component,
  Input,
  Output,
} from "@angular/core";
import {
  ControlValueAccessor,
  FormControl,
  NG_VALUE_ACCESSOR,
} from "@angular/forms";
import {MatSelectChange} from "@angular/material";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import _ from "lodash";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  FormValidationHelper,
  isEmptyArray,
  isNullOrUndefined,
  ObservableUtil,
  StringUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-multi-select",
  templateUrl: "./shared-multi-select.presentational.html",
  styleUrls: ["./shared-multi-select.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: SharedMultiSelectPresentational,
    },
  ],
})
export class SharedMultiSelectPresentational extends SharedAbstractPresentational implements ControlValueAccessor {
  @Input()
  placeholder: string;

  @Input()
  list: string[];

  @Input()
  valueKey: string;

  @Input()
  labelKey: string;

  @Input()
  formControl: FormControl;

  @Input()
  required: boolean;

  get formValidationHelper(): typeof FormValidationHelper {
    return FormValidationHelper;
  }

  private readonly _valueBS: BehaviorSubject<any[] | undefined> = new BehaviorSubject<any[] | undefined>(undefined);

  @Output("valueChange")
  readonly valueObs: Observable<any[] | undefined> = ObservableUtil.asObservable(this._valueBS);

  constructor() {
    super();
  }

  getCorrespondingLabel(valueElement: string | undefined): string {
    if (valueElement === undefined || isNullOrUndefined(this.list) || isEmptyArray(this.list)) {
      return StringUtil.stringEmpty;
    }
    return this.list.find(v => v[this.valueKey] === valueElement)[this.labelKey];
  }

  getCorrespondingLabelForOthers(): string {
    const listOtherValue = _.slice(this.formControl.value, 1);
    const labelOther: string[] = [];
    listOtherValue.forEach(o => labelOther.push(this.getCorrespondingLabel(o)));
    return _.join(labelOther, ", ");
  }

  selectionChange($event: MatSelectChange): void {
    this.propagateChange($event.value);
    this._valueBS.next($event.value);
  }

  propagateChange = (__: any) => {};

  registerOnChange(fn: any): void {
    this.propagateChange = fn;
  }

  registerOnTouched(fn: any): void {
  }

  setDisabledState(isDisabled: boolean): void {
  }

  writeValue(value: string[]): void {
    // if (value) {
    // this.formControl.setValue(value);
    // }
  }
}
